@@ -15,7 +15,7 @@ def main(*args):
             "available on your PYTHONPATH environment variable? Did you "
             "forget to activate a virtual environment?"
         ) from exc
-    execute_from_command_line(*args)
+    execute_from_command_line(args)
 
 
 if __name__ == '__main__':
